#ifndef MYX_REDIS_CLIENT_HPP_
#define MYX_REDIS_CLIENT_HPP_

#pragma once

#include <myx/base/config.hpp>
#include <myx/redis/lexer.hpp>
#include <myx/redis/parser.hpp>
#include <myx/redis/request.hpp>

#include <QQueue>
#include <QScopedPointer>
#include <QTcpSocket>

namespace myx {

namespace redis {

/**
 * @brief Provides access to a Redis server
 */
class Client : public QObject
{
	Q_OBJECT

public:

	/**
	 * @brief Creates a Redis client
	 * @param parent the parent QObject
	 */
	explicit Client( QObject* parent = nullptr );

	/**
	 * @brief Destroys the client
	 */
	~Client() override = default;

	/*
	 * Note: we specifically avoid an overload of connectToHost that
	 * uses QHostAddress since that would force anyone using the client
	 * library to use the QtNetwork module, which we wish to avoid.
	 */

	/**
	 * @brief Attempts to connect to the specified Redis server
	 * @param hostName the hostname of the Redis server
	 * @param port the port that the Redis server is listening on
	 *
	 * If the connection was successful, the connected() signal will be
	 * emitted.
	 */
	void connectToHost( const QString& hostName, quint16 port = 6379 );

	/**
	 * @brief Disconnects from the Redis server
	 */
	void disconnectFromHost();

	/**
	 * @brief Indicates whether the client is connected to a Redis server
	 * @return true if the client is connected
	 */
	bool isConnected() const;

	/**
	 * @brief Sends the specified command to the Redis server
	 * @param command the command to execute
	 * @return an object representing the request
	 */
	Request* sendCommand( const QByteArray& command );

	/**
	 * @brief Attempts to set the specified key to the specified value
	 * @param name the name of the key
	 * @param value the value of the key
	 * @return the request issued
	 */
	Request* set( const QByteArray& name, const QByteArray& value );

	/**
	 * @brief Waits for the socket to finish connecting
	 * @param msecs the amount of time in milliseconds to wait
	 * @return true if the connection was completed
	 *
	 * Note: to wait indefinitely, pass a value of -1.
	 */
	bool waitForConnected( int msecs = 30000 );

	/**
	 * @brief Waits for the socket to finish disconnecting
	 * @param msecs the amount of time in milliseconds to wait
	 * @return true if the disconnection was completed
	 */
	bool waitForDisconnected( int msecs = 30000 );

	/**
	 * @brief Emitted when the client establishes a connection with the Redis server
	 */
	Q_SIGNAL void connected();

	/**
	 * @brief Emitted when the client disconnects from the Redis server
	 */
	Q_SIGNAL void disconnected();

private:

	class ClientPrivate : public QObject
	{
		Q_OBJECT

		friend class Client;

		explicit ClientPrivate( Client* client = nullptr ) :
			m_lexer ( &m_socket ),
			m_parser( &m_lexer )
		{
			connect( &m_socket, &QTcpSocket::connected, client, &Client::connected );
			connect( &m_socket, &QTcpSocket::disconnected, client, &Client::disconnected );
			connect( &m_parser, &Parser::reply, this, &ClientPrivate::sendReply );
		}


		QTcpSocket         m_socket;
		QQueue< Request* > m_queue;
		Lexer              m_lexer;
		Parser             m_parser;

		Q_SLOT void sendReply( const myx::redis::Reply& reply )
		{
			Q_EMIT m_queue.dequeue()->reply( reply );
		}
	}; // class ClientPrivate

	const QScopedPointer< ClientPrivate > d;
}; // class Client

} // namespace redis

} // namespace myx

#ifdef MYXLIB_HEADER_ONLY
#include "client-inl.hpp"
#endif


#endif // ifndef MYX_REDIS_CLIENT_HPP_
